/* deeppathstore.hh
 * This file belongs to Worker, a file manager for UN*X/X11.
 * Copyright (C) 2012 Ralf Hoffmann.
 * You can contact me at: ralf@boomerangsworld.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef PATHSTORE
#define PATHSTORE

#include <list>
#include <string>

class DeepPathNode
{
public:
    DeepPathNode( const std::string &name ) : m_name( name )
    {
    }

    DeepPathNode()
    {
    }

    const std::string &get_name() const
    {
        return m_name;
    }

    const std::list< DeepPathNode > &get_children() const
    {
        return m_children;
    }

    DeepPathNode &lookup_or_insert( const std::string &name, bool *changed )
    {
        for ( std::list< DeepPathNode >::iterator it1 = m_children.begin();
              it1 != m_children.end();
              it1++ ) {
            if ( it1->get_name() == name ) return *it1;
        }

        if ( changed != NULL ) {
            *changed = true;
        }

        return *m_children.insert( m_children.end(), name );
    }
private:
    std::list< DeepPathNode > m_children;
    std::string m_name;
};

class DeepPathStore
{
public:
    DeepPathStore();

    void storePath( const std::string &path, bool *changed = NULL );
    std::list< std::string > getPaths();
    void clear();
private:
    void storePath( DeepPathNode &node,
                    const std::string &path,
                    std::string::size_type start_pos,
                    bool *changed );

    DeepPathNode m_root;
};

#endif
